//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/View/Instrument/OffspecBeamEditor.cpp
//! @brief     Implements class OffspecBeamEditor
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#include "GUI/View/Instrument/OffspecBeamEditor.h"
#include "GUI/Model/Descriptor/DoubleDescriptor.h"
#include "GUI/Model/Device/BeamAngleItems.h"
#include "GUI/Model/Device/BeamWavelengthItem.h"
#include "GUI/Model/Device/InstrumentItems.h"
#include "GUI/View/Common/FixupDoubleValidator.h"
#include "GUI/View/Instrument/AxisPropertyEditor.h"
#include "GUI/View/Instrument/DistributionEditor.h"
#include "GUI/View/Tool/GroupBoxCollapser.h"
#include <QDoubleValidator>
#include <QFormLayout>
#include <QGridLayout>
#include <QLineEdit>

OffspecBeamEditor::OffspecBeamEditor(QWidget* parent, OffspecInstrumentItem* item)
    : QGroupBox("Beam parameters", parent)
{
    ASSERT(item);

    auto* vLayout = new QVBoxLayout(this);
    vLayout->setContentsMargins(30, 8, 0, 0);
    auto* form = new QFormLayout();
    form->setFieldGrowthPolicy(QFormLayout::FieldsStayAtSizeHint);
    vLayout->addLayout(form);
    auto* intensityEditor = new QLineEdit(this);
    intensityEditor->setToolTip("Beam intensity in neutrons (or gammas per second.");
    auto* validator = new FixupDoubleValidator(intensityEditor);
    validator->setNotation(QDoubleValidator::ScientificNotation);
    validator->setRange(0.0, 1e+32, 1000);
    intensityEditor->setValidator(validator);
    form->addRow("Intensity:", intensityEditor);

    auto* wavelengthEditor =
        new DistributionEditor("Wavelength", MeanConfig{true}, GUI::ID::Distributions::All, this,
                               item->beamItem()->wavelengthItem());
    vLayout->addWidget(wavelengthEditor);

    auto* inclinationEditor =
        new AxisPropertyEditor(this, "Inclination angle", &item->alphaAxis(),
                               "Number of points in scan");
    vLayout->addWidget(inclinationEditor);

    auto* azimuthalEditor =
        new DistributionEditor("Azimuthal angle", MeanConfig{false}, GUI::ID::Distributions::All,
                               this, item->beamItem()->azimuthalAngleItem());
    vLayout->addWidget(azimuthalEditor);

    intensityEditor->setText(QString::number(item->beamItem()->intensity()));

    GroupBoxCollapser::installIntoGroupBox(this);

    connect(wavelengthEditor, &DistributionEditor::distributionChanged, this,
            &OffspecBeamEditor::dataChanged);
    connect(inclinationEditor, &AxisPropertyEditor::dataChanged, this,
            &OffspecBeamEditor::dataChanged);
    connect(azimuthalEditor, &DistributionEditor::distributionChanged, this,
            &OffspecBeamEditor::dataChanged);

    // validate value while typing
    connect(intensityEditor, &QLineEdit::textEdited, [=]() {
        QString str = intensityEditor->text();
        int pos;
        if (intensityEditor->validator()->validate(str, pos) == QValidator::Acceptable) {
            item->beamItem()->setIntensity(intensityEditor->text().toDouble());
            emit dataChanged();
        }
    });
    // if validator does not assert the input value, it says about that,
    // and here we return to the last correct value
    connect(validator, &FixupDoubleValidator::fixupSignal, [=]() {
        auto* editor = qobject_cast<QLineEdit*>(validator->parent());
        editor->setText(QString::number(item->beamItem()->intensity(), 'g'));
    });
}
